/*  VECTOR.H

    Vector Library Include File
    Declarations for 2- and 3-dim VECTORs.

    Copyright (c) 1993-2001 by OctoNus Ltd.
    All rights reserved.
*/

#ifndef _INC_Vector_3A262AD20271_INCLUDED
#define _INC_Vector_3A262AD20271_INCLUDED

#ifdef VECTOR_EXPORTS
# define VECTORDLL_API
#else
# define VECTORDLL_API _declspec (dllimport)
#endif

#include <math.h> /* sqrt() in inline functions */

struct  VECTOR
{
  // -------- Implementation -------- //
        double    x, y, z;
  // -------------------------------- //

  // -- Constructors
        VECTOR  ( );
        VECTOR  ( const VECTOR & );
        VECTOR  (double X, double Y, double Z) { x=X; y=Y; z=Z; }

  // -- Indexed access operator
        double &          operator [] ( int );
        const double &    operator [] ( int ) const;

  // -- Addition & Subtraction operators
        VECTOR      operator +  ( ) const { return *this; }
        VECTOR      operator -  ( ) const { return VECTOR (-x,-y,-z); }
        VECTOR &    operator += (const VECTOR &v) { x+=v.x; y+=v.y; z+=v.z; return *this; }
        VECTOR &    operator -= (const VECTOR &v) { x-=v.x; y-=v.y; z-=v.z; return *this; }

  // -- Multiplication by double number operators
        VECTOR &    operator *= (double r) { x*=r; y*=r; z*=r; return *this; }
        VECTOR &    operator /= (double r) { x/=r; y/=r; z /=r; return *this; }

  // -- Miscellaneous operators & functions
        double        abs ( ) const { return sqrt (x*x+y*y+z*z); }
        VECTOR      fix (double r = 1) const;
        VECTOR      project (const VECTOR &v) const;
};


typedef VECTOR  VECTOR3;
typedef VECTOR  POINT3;

// ------------------------------------ //
//  Out-of-class operators & functions  //
// ------------------------------------ //

inline int  operator== (const VECTOR &v1, const VECTOR &v2)
{ return v1.x==v2.x && v1.y==v2.y && v1.z==v2.z; }
inline int  operator!= (const VECTOR &v1, const VECTOR &v2)
{ return v1.x!=v2.x || v1.y!=v2.y || v1.z !=v2.z; }
inline VECTOR  operator+ (const VECTOR &v1, const VECTOR &v2)
{ return VECTOR (v1.x+v2.x, v1.y+v2.y, v1.z+v2.z); }
inline VECTOR  operator- (const VECTOR &v1, const VECTOR &v2)
{ return VECTOR (v1.x-v2.x, v1.y-v2.y, v1.z-v2.z); }
inline VECTOR  operator* (const VECTOR &v, double r)
{ return VECTOR (v.x*r, v.y*r, v.z*r); }
inline VECTOR  operator/ (const VECTOR &v, double r)
{ return VECTOR (v.x/r, v.y/r, v.z/r); }
inline VECTOR  operator* (double r, const VECTOR &v)
{ return VECTOR (v.x*r, v.y*r, v.z*r); }
inline double    operator |  ( const VECTOR &v1 , const VECTOR &v2 ) 
{ return v1.x*v2.x + v1.y*v2.y + v1.z*v2.z; }
inline VECTOR  operator& (const VECTOR &v1, const VECTOR &v2)
{
    VECTOR res;
    res.x = v1.y*v2.z-v1.z*v2.y;
    res.y = v1.z*v2.x-v1.x*v2.z;
    res.z = v1.x*v2.y-v1.y*v2.x;
    return res;
}
inline double  abs (const VECTOR &v)
{ return sqrt (v.x*v.x+v.y*v.y+v.z*v.z); }


VECTORDLL_API const VECTOR &  getZeroVector ( );

#define VECTOR_0 (getZeroVector ( ))
#define POINT_0 VECTOR_0

// ------------------------------------------------ //
//  Inline definitions for some 'dummy' functions   //
// ------------------------------------------------ //


inline  VECTOR:: VECTOR () {}

inline  VECTOR:: VECTOR (VECTOR const &v) { *this=v; }

inline  double & VECTOR:: operator [] ( int index )
{
    return  ( ( double * ) ( void * ) this ) [ index ];
}


inline  const double & VECTOR:: operator [] ( int index ) const
{
    return  ( ( const double * ) ( const void * ) this ) [ index ];
}

extern void supplement_reper ( const VECTOR & norm, VECTOR & ax, VECTOR & ay );


// 2-dim Vector 
class   VECTOR2
{
  // --------- Data members --------- //
    public:
        double    x, y;
  // -------------------------------- //

    public:

  // -- Constructors
        VECTOR2 ( );
        VECTOR2 ( const VECTOR2 & );
        VECTOR2 ( double , double );

  // -- Indexed access operator
        double &          operator [] ( int );
        const double &    operator [] ( int ) const;

  // -- Addition & Subtraction operators
        VECTOR2         operator +  ( ) const;
        VECTOR2         operator -  ( ) const;
        VECTOR2 &       operator += ( const VECTOR2 & );
        VECTOR2 &       operator -= ( const VECTOR2 & );

  // -- Multiplication by double number operators
        VECTOR2 &       operator *= ( double );
        VECTOR2 &       operator /= ( double );

  // -- Miscellaneous operators & functions
        double    abs ( ) const;
        VECTOR2 fix ( double = 1 ) const;
        VECTOR2 project ( const VECTOR2 & ) const;

  // -- Out-of-class operators & functions

friend  int operator == ( const VECTOR2 & , const VECTOR2 & );
friend  int operator != ( const VECTOR2 & , const VECTOR2 & );
friend  VECTOR2 operator +  ( const VECTOR2 & , const VECTOR2 & );
friend  VECTOR2 operator -  ( const VECTOR2 & , const VECTOR2 & );
friend  VECTOR2 operator *  ( const VECTOR2 & , double );
friend  VECTOR2 operator /  ( const VECTOR2 & , double );
friend  VECTOR2 operator *  ( double , const VECTOR2 & );
friend  double    operator |  ( const VECTOR2 & , const VECTOR2 & );
friend  double    operator &  ( const VECTOR2 & , const VECTOR2 & );

friend  double    abs ( const VECTOR2 & );

};

typedef VECTOR2 POINT2;

VECTORDLL_API const VECTOR2 & getZeroVector2 ( );
#define VECTOR2_0 (getZeroVector2 ( ))

// ----------------------------- //
//  Inline function definitions  //
// ----------------------------- //

inline  double &  VECTOR2:: operator [] ( int index )
{
    return  ( ( double * ) ( void * ) this ) [ index ];
}


inline  const double &  VECTOR2:: operator [] ( int index ) const
{
    return  ( ( const double * ) ( const void * ) this ) [ index ];
}

extern double polygonArea ( const VECTOR2 * v, int n );


class   LINE2
{
    public:
  // -------- Implementation -------- //
        VECTOR2 norm;
        double    dist;
  // -------------------------------- //

  // -- Constructors
        LINE2   ( );
        LINE2   ( const VECTOR2 & , double );
        LINE2   ( const VECTOR2 & , const VECTOR2 & );
};

struct TRIANGLE2
{
    LINE2 a, b, c;
};

typedef TRIANGLE2 * P_TRIANGLE;


struct CIRCLES2
{
    VECTOR2 center;
    double    radius;
};

struct POLYGON2
{
    VECTOR2 * vertex;
    LINE2 * side;
    int convex;
    TRIANGLE2 * triangle;
    int nv, nt;
};

extern int threeangulation ( TRIANGLE2 * dst, const VECTOR2 * V, int nv );
extern int is_convex ( const POLYGON2 & p );

//
//           ,   
//          
extern VECTOR2 lines_intersect ( const LINE2 & l1, const LINE2 & l2 );


// Plane

struct  PLANE
{
  // -------- Implementation -------- //
        VECTOR3 norm;
        double    dist;
  // -------------------------------- //

  // -- Constructors
        PLANE   ( );
        PLANE   ( const VECTOR3 & , double );

        PLANE   operator -  ( ) const;

};


struct  MATRIX;

struct  SPIN
{
    public:
  // -------- Implementation -------- //
        double  t, x, y, z;
  // -------------------------------- //

    public:

  // -- Constructors
        SPIN    ( );
        SPIN    ( const SPIN & );
        SPIN    ( const VECTOR3 & , double );

  // -- Operators
        SPIN    operator ~  ( ) const;
        VECTOR3 operator () ( const VECTOR3 & ) const;
        MATRIX  operator () ( const MATRIX & ) const;

};


// ------------------------------------ //
//  Out-of-class operators & functions  //
// ------------------------------------ //

SPIN    operator *  ( const SPIN & , const SPIN & );

extern void getspin ( SPIN & s, const VECTOR3 & a, const VECTOR3 & b, const VECTOR3 * n );

VECTORDLL_API const SPIN & getIdentitySpin ();

#define SPIN_1 (getIdentitySpin ())



// The Matrix
struct  MATRIX
{
  // -------- Implementation -------- //
        VECTOR3 x, y, z;
  // -------------------------------- //

  // -- Constructors
        MATRIX  ( );
        MATRIX  ( const MATRIX & );
        MATRIX  ( const SPIN & );
        MATRIX  ( const VECTOR3 & , const VECTOR3 & , const VECTOR3 & );
        MATRIX  (
                    double , double , double ,
                    double , double , double ,
                    double , double , double
                );

  // -- Indexed access operator
        VECTOR3 &       operator [] ( int );
        const VECTOR3 & operator [] ( int ) const;

  // -- Addition & Subtraction operators
        MATRIX      operator +  ( ) const;
        MATRIX      operator -  ( ) const;
        MATRIX &    operator += ( const MATRIX & );
        MATRIX &    operator -= ( const MATRIX & );

  // -- Multiplication by double number operators
        MATRIX &    operator *= ( double );
        MATRIX &    operator /= ( double );

  // -- Miscellaneous operators & functions
        MATRIX      operator ~  ( ) const;
        MATRIX      operator !  ( ) const;
        double        I       ( ) const;
        double        II      ( ) const;
        double        III     ( ) const;
        double        tr      ( ) const;
        double        det     ( ) const;
        MATRIX      conj    ( ) const;
        MATRIX      comp    ( ) const;
        MATRIX      inv     ( ) const;
        VECTOR3     col     ( int ) const;
        VECTOR3     row     ( int ) const;

};


// ------------------------------------ //
//  Out-of-class operators & functions  //
// ------------------------------------ //

int     operator == ( const MATRIX  & , const MATRIX & );
int     operator != ( const MATRIX  & , const MATRIX & );
MATRIX  operator +  ( const MATRIX  & , const MATRIX & );
MATRIX  operator -  ( const MATRIX  & , const MATRIX & );
MATRIX  operator *  ( const MATRIX  & , double );
MATRIX  operator /  ( const MATRIX  & , double );
MATRIX  operator *  ( double , const MATRIX & );
VECTOR3 operator |  ( const MATRIX  & , const VECTOR3 & );
VECTOR3 operator |  ( const VECTOR3 & , const MATRIX  & );
MATRIX  operator |  ( const MATRIX  & , const MATRIX  & );
MATRIX  operator &  ( const MATRIX  & , const VECTOR3 & );
MATRIX  operator &  ( const VECTOR3 & , const MATRIX  & );
MATRIX  operator ^  ( const VECTOR3 & , const VECTOR3 & );

double    tr      ( const MATRIX & );
double    det     ( const MATRIX & );


VECTORDLL_API const MATRIX & getZeroMatrix ();

#define MATRIX_0 (getZeroMatrix ())


// ------------------------------------------------ //
//  Inline definitions for some 'dummy' functions   //
// ------------------------------------------------ //


inline  VECTOR3 &  MATRIX:: operator [] ( int index )
{
    return  ( ( VECTOR3 * ) ( void * ) this ) [ index ];
}


inline  const VECTOR3 &  MATRIX:: operator [] ( int index ) const
{
    return  ( ( const VECTOR3 * ) ( const void * ) this ) [ index ];
}

class   CONFORM
{
    public:
  // -------- Implementation -------- //
        SPIN    spin;
        VECTOR3 trans;
        double    magn;
  // -------------------------------- //

  // -- Constructor
        CONFORM ( const SPIN & , const VECTOR3 & , double );

  // -- Operators
        CONFORM operator ~ ( ) const;
        PLANE   operator () ( const PLANE & ) const;

};

class   BASIS :
public  CONFORM
{
    public:
        int alt; // BasisFlagValue flags, see the list below

        BASIS ( );
        BASIS ( const CONFORM & , int = 0 );
        BASIS ( const SPIN &, const VECTOR3 &, double , int = 0 );

};

          // Value of BASIS::alt can be a combination of the following:
enum BasisFlagValue {
    RM_NOTHING = 0x00,
    RM_ROTATE  = 0x01,
    RM_MOVE    = 0x02,
    RM_OBJECT  = 0x04,
    RM_SIDE    = 0x08,
    RM_IMAGE   = 0x10,
    RM_ALL     = 0x1F,
    RM_ONLY    = 0x20
};

// ------------------------------------ //
//  Out-of-class operators & functions  //
// ------------------------------------ //
BASIS operator >> ( const BASIS & , const BASIS & );
BASIS operator >> ( const SPIN & , const BASIS & );
BASIS operator >> ( const VECTOR3 & , const BASIS & );
BASIS operator >> ( const double & , const BASIS & );

VECTORDLL_API const BASIS & getIdentityBasis ();
#define BASIS_1 (getIdentityBasis ())

struct  SIMILARITY
{
  // -------- Implementation -------- //
        MATRIX  rot;
        VECTOR3 trans;
  // -------------------------------- //

  // -- Constructors
        SIMILARITY ( );
        SIMILARITY ( const CONFORM & );

  // -- Operators

        POINT3  operator () ( const POINT3 & ) const;

};


#endif  //  _INC_Vector_3A262AD20271_INCLUDED

